LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY Receiver_unit IS
	PORT (	
		CLK			: 	IN  STD_LOGIC;	
		CD			:	IN  STD_LOGIC;
		Enable_RX	:	IN  STD_LOGIC;
	   	Serial_in   :	IN  STD_LOGIC;
		RX_OUT		:	OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		Error		:	OUT STD_LOGIC;
		New_Data	:	OUT STD_LOGIC
	);
END Receiver_unit;   
		

ARCHITECTURE structural OF Receiver_unit IS

COMPONENT receiver_control_unit IS
   	PORT (
			CLK 			: IN	STD_LOGIC;
			CD 				: IN	STD_LOGIC;
			Start_Bit		: IN	STD_LOGIC;
			Enable_RX		: IN	STD_LOGIC;
			TC4				: IN    STD_LOGIC;
			TC8				: IN	STD_LOGIC;
			TC10			: IN	STD_LOGIC;
			EP_Check		: IN    STD_LOGIC;
			Stop_Bit		: IN	STD_LOGIC;
			S				: OUT   STD_LOGIC_VECTOR (1 DOWNTO 0);
			LDD				: OUT	STD_LOGIC;
			CE_Mod4			: OUT   STD_LOGIC;
			CE_Mod8			: OUT	STD_LOGIC;	
			Clear_Counters	: OUT	STD_LOGIC;
			Error			: OUT	STD_LOGIC;
			New_Data		: OUT	STD_LOGIC
		);
END COMPONENT;   

COMPONENT Receiver_datapath IS
	PORT (	
		Serial_in		:	IN STD_LOGIC;
	   	S				:	IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		LDD				:	IN STD_LOGIC;
		CE_Mod4			:	IN STD_LOGIC;
		Clear_Counters	:	IN STD_LOGIC;		
		CE_Mod8			:	IN STD_LOGIC;
		CD				:	IN STD_LOGIC;
		CLK				:	IN STD_LOGIC;
		RX_OUT			:	OUT STD_LOGIC_VECTOR(7 DOWNTO 0);	
		EP_Check		:	OUT STD_LOGIC;
		Stop_Bit		:	OUT STD_LOGIC;
		TC4				:	OUT STD_LOGIC;
		TC8				:	OUT STD_LOGIC;
		TC10			:	OUT STD_LOGIC
		);
END COMPONENT;

	SIGNAL Start_Bit 		: STD_LOGIC;
	SIGNAL Stop_Bit			: STD_LOGIC; 
	SIGNAL EP_Check 		: STD_LOGIC;
	SIGNAL TC8 				: STD_LOGIC;
	SIGNAL TC4				: STD_LOGIC; 
	SIGNAL TC10 			: STD_LOGIC;
	SIGNAL LDD 				: STD_LOGIC;
	SIGNAL S				: STD_LOGIC_VECTOR (1 DOWNTO 0); 
	SIGNAL CE_Mod4 			: STD_LOGIC;
	SIGNAL CE_Mod8 			: STD_LOGIC;	
	SIGNAL Clear_Counters 	: STD_LOGIC;

BEGIN

Chip1 : receiver_control_unit
	PORT MAP (
		Enable_RX		=> Enable_RX,
		Start_Bit 		=> Start_Bit,
		Stop_Bit 		=> Stop_Bit,
		EP_Check 		=> EP_Check,
		TC4 			=> TC4,
		TC8 			=> TC8,
		TC10 			=> TC10,
		CD 				=> CD,
		CLK 			=> CLK,
		LDD 			=> LDD,
		S 				=> S,
		CE_Mod4 		=> CE_Mod4,
		CE_Mod8 		=> CE_Mod8,
		Clear_Counters 	=> Clear_Counters,
		Error 			=> Error,
		New_Data 		=> New_Data
	);

Chip2 : Receiver_datapath
	PORT MAP (
		Serial_in 		=> Serial_in,
		LDD				=> LDD,
		S 				=> S,
		CE_Mod4 		=> CE_Mod4,
		CE_Mod8 		=> CE_Mod8,
		Clear_Counters 	=> Clear_Counters,
		CD 				=> CD,
		CLK 			=> CLK,
		RX_OUT 			=> RX_OUT,
		Stop_Bit 		=> Stop_Bit,
		EP_Check 		=> EP_Check,
		TC4 			=> TC4,
		TC8 			=> TC8,
		TC10 			=> TC10
	);

-- Extra logic 
Start_Bit <= Serial_in;


END structural ;