-- Example of a binary synchronous counter 
--  using FSM style 
-- Baud rate  9600 b/s
-- Board DE0 50 MHz crystall clock	 	--> divider by 651 	-->  Max_Count 560 = 1010001010   --> 10 bits
-- Board UP2 25.175 MHz crystall clock 	--> divider by 328  -->  Max_Count 327 = 101000111   --> 9 bits


-- Baud rate  110 b/s
-- Board DE0 50 MHz crystall clock --> divider by 56818		-->	 Max_Count 56817 = 1101110111110001   --> 16 bits
-- Board UP2 25.175 MHz crystall clock --> divider by 28608	-->  Max_Count 28607 = 110111110111111   --> 15 bits


LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY RECEIVER_CLOCK_DIVIDER IS
		PORT(	CD,CLK,CE		: 	IN  std_logic;
				TC651			:	OUT std_logic
		);                 
END RECEIVER_CLOCK_DIVIDER;

			-- Internal desciption in FSM style

ARCHITECTURE FSM_like OF RECEIVER_CLOCK_DIVIDER IS



CONSTANT Max_Count	: STD_LOGIC_VECTOR(9 DOWNTO 0) := "1010001010"; -- 560 en binari 
CONSTANT Reset		: STD_LOGIC_VECTOR(9 DOWNTO 0) := "0000000000"; 

-- Internal wires 

SIGNAL present_state,future_state: STD_LOGIC_VECTOR(9 downto 0):= "0000000000" ;   
                       										-- This initialisation only make sense for the simulator  

BEGIN
------------------------------ the only clocked block : the state register
state_register: PROCESS (CD, CLK)
	BEGIN
						
	IF CD = '1' THEN 						-- reset counter
		present_state <= Reset; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
		present_state <= future_state; 
	END IF;

END PROCESS state_register;

------------------------- CC1. combinational system for calculating next state
CS_1: PROCESS (present_state, CE)
	BEGIN
	IF CE = '1' THEN
		IF(present_state < Max_Count ) THEN
			future_state <= present_state + 1 ;
		ELSE
			future_state <= Reset;
		END IF;
	ELSE 	
		future_state <= present_state;  -- count disable
	END IF;

END PROCESS CS_1;

------------------------- CS_2: combinational system for calculating output logic  

TC651 <= '1' WHEN ((present_state = Max_count)) ELSE '0';


END FSM_like;






